@@ -14,6 +14,9 @@
 #define GLOBAL_CONFIG_FILE "/opt/cloudgps/cloudgps.ini"
 #define PROVIDERS_CONFIG_FILE "/opt/cloudgps/providers.ini"
 
+
+#include "file.h"
+
 char *GROUP_DEFAULTS = "Defaults";
 void loadTileProviders() {
     GKeyFile *keyfile;
@@ -23,7 +26,7 @@ void loadTileProviders() {
     keyfile = g_key_file_new();
 
     if (!g_key_file_load_from_file(keyfile, PROVIDERS_CONFIG_FILE, 0, &error)) {
-        fprintf(stderr, error->message);
+        fprintf(stderr, "%s", error->message);
         exit(1);
     }
     gchar** names = g_key_file_get_groups(keyfile, &numGroups);
@@ -34,10 +37,18 @@ void loadTileProviders() {
         TileProvider *provider = calloc(1, sizeof(TileProvider));
         provider -> name = names[i];
         provider -> filenameFormat = g_key_file_get_string(keyfile, names[i], "fileFormat", NULL);
+        provider -> urlFormatType = g_key_file_get_string(keyfile, names[i], "urlFormatType", NULL);
+        gchar* quadKeyFirstChar = g_key_file_get_string(keyfile, names[i], "quadKeyFirstChar", NULL);
+        if (quadKeyFirstChar) {
+        	provider -> quadKeyFirstChar = quadKeyFirstChar[0];
+        } else {
+        	provider -> quadKeyFirstChar = '0';
+        }
         provider -> urlFormat = g_key_file_get_string(keyfile, names[i], "urlFormat", NULL);
         provider -> minZoom = g_key_file_get_integer(keyfile, names[i], "minZoom", NULL);
         provider -> maxZoom = g_key_file_get_integer(keyfile, names[i], "maxZoom", NULL);
-        //fprintf(stderr, "provider name = '%s', fileFormat = '%s', urlFormat = '%s', minZoom = %d, maxZoom =%d\n", provider -> name, provider -> filenameFormat, provider -> urlFormat, provider -> minZoom, provider -> maxZoom);
+        fprintf(stderr, "provider name = '%s', fileFormat = '%s', urlFormat = '%s', urlFormatType='%s', quadKeyFirstChar='%c', minZoom = %d, maxZoom =%d\n",
+        		provider -> name, provider -> filenameFormat, provider -> urlFormat, provider -> urlFormatType,provider -> quadKeyFirstChar, provider -> minZoom, provider -> maxZoom);
         tileProviders = g_list_append(tileProviders, provider);
         i++;
     }
@@ -67,16 +78,18 @@ void loadConfig() {
         options.locallyLoadedFadeTime = g_key_file_get_integer(keyfile, GROUP_DEFAULTS, "locallyLoadedFadeTime", NULL);
         canvas.rotation2d = g_key_file_get_integer(keyfile, GROUP_DEFAULTS, "rotation2d", NULL);
         ttfFont = g_key_file_get_string(keyfile, GROUP_DEFAULTS, "ttfFont", NULL);
-        font=TTF_OpenFont(ttfFont, 16);
-        if(!font) {
-            fprintf(stderr, "TTF_OpenFont: %s\n", TTF_GetError());
-            exit(1);
-        }
+        //font=TTF_OpenFont(ttfFont, 16);
+//        if(!font) {
+//            fprintf(stderr, "TTF_OpenFont: %s\n", TTF_GetError());
+            //exit(1);
+//        }
     } else {
         fprintf(stderr, "Error loading %s %s\n", GLOBAL_CONFIG_FILE, error->message);
-        canvas.zoom = 3;
-        canvas.tilex = -1 - getCanvasShiftX();
-        canvas.tiley = -1 - getCanvasShiftY();
+        canvas.zoom = 2;
+        canvas.x = 0;
+        canvas.y = 0;
+        canvas.tilex = -1;
+        canvas.tiley = -1;
         options.mipmapToggle = 1;
         options.showDebugTiles = 0;
         options.downloadedFadeTime = 1500;
@@ -143,7 +156,7 @@ void saveConfig() {
     } else {
         FILE* file = fopen(USER_CONFIG_FILE, "wt");
         if (file != NULL) {
-            fprintf(file, data);
+            fprintf(file, "%s", data);
             fclose(file);
         } else {
             fprintf(stderr, "Unable to open %s for writing.\n", USER_CONFIG_FILE);
