@@ -11,7 +11,10 @@
  * Author: Damian Waradzyn
  */
 
+#include "downloadQueue.h"
+
 SDL_mutex *mutex = NULL;
+SDL_sem *sem = NULL;
 
 GQueue* createQueue() {
     GQueue* queue = g_queue_new();
@@ -22,6 +25,13 @@ GQueue* createQueue() {
     if (mutex == NULL) {
         mutex = SDL_CreateMutex();
     }
+    if (sem == NULL) {
+        sem = SDL_CreateSemaphore(0);
+        if (sem == NULL) {
+        	fprintf(stderr, "SDL_CreateSemaphore\n");
+        	exit(1);
+        }
+    }
 //    fprintf(stderr, "createQueue success\n");
     return queue;
 }
@@ -31,13 +41,30 @@ void enqueue(GQueue* queue, gpointer data) {
         fprintf(stderr, "mutexP failed in enqueue\n");
         return;
     }
+//#define DISABLE_NON_ZOOM_TILES
+#ifdef DISABLE_NON_ZOOM_TILES
+    t_tile* tile = (t_tile*) data;
+    if (tile -> zoomHelper == 0) {
+		if (SDL_mutexV(mutex) == -1) {
+			fprintf(stderr, "mutexV failed in enqueue\n");
+		}
+        fprintf(stderr, "skipping non helper tile\n");
+    	return;
+    }
+#endif
     g_queue_push_tail(queue, data);
+    if (SDL_SemPost(sem) == -1) {
+        fprintf(stderr, "SemPost failed in enqueue\n");
+        return;
+    }
     if (SDL_mutexV(mutex) == -1) {
         fprintf(stderr, "mutexV failed in enqueue\n");
     }
 }
 
+long lastMinPriorityDequeue = 0;
 gpointer dequeue(GQueue* queue) {
+//	long start = SDL_GetTicks();
     if (queue -> length == 0) {
         return NULL;
     }
@@ -56,7 +83,7 @@ gpointer dequeue(GQueue* queue) {
     GList * elem = queue -> head;
     while (elem != NULL) {
         t_tile * tile = (t_tile*) elem -> data;
-        if (tile -> visible) {
+        if (tile -> zoomHelper) {
             data = elem->data;
             g_queue_remove_all(queue, tile);
             break;
@@ -64,15 +91,41 @@ gpointer dequeue(GQueue* queue) {
         elem = elem -> next;
     }
 
+    if (data == NULL) {
+		elem = queue -> head;
+        while (elem != NULL) {
+            t_tile * tile = (t_tile*) elem -> data;
+            if (tile -> visible) {
+                data = elem->data;
+                g_queue_remove_all(queue, tile);
+                break;
+            }
+            elem = elem -> next;
+        }
+    }
     if (data == NULL) {
         // TODO return tile nearest to visible tile
         data = g_queue_pop_head(queue);
     }
+	if (SDL_mutexV(mutex) == -1) {
+		fprintf(stderr, "mutexV failed in dequeue(2)\n");
+	}
 
-    if (SDL_mutexV(mutex) == -1) {
-        fprintf(stderr, "mutexV failed in dequeue(2)\n");
+    if (data != NULL) {
+    	t_tile* tile = (t_tile*)data;
+    	tile -> state = STATE_LOADING;
+    	if (tile -> zoomHelper == 0 && tile -> visible == 0) {
+    		long tmp = SDL_GetTicks() - lastMinPriorityDequeue;
+    		lastMinPriorityDequeue = SDL_GetTicks();
+    		if (tmp < 500) {
+    			SDL_Delay(500);
+    		}
+    	}
     }
-    ((t_tile*)data) -> state = STATE_LOADING;
+//    long end = SDL_GetTicks();
+//    if (end - start > 1) {
+//		fprintf(stderr, "dequeue took %d ms\n", end - start);
+//    }
     return data;
 }
 
