@@ -10,6 +10,12 @@
  *
  * Author: Damian Waradzyn
  */
+
+#include "console.h"
+
+extern int downloaded;
+extern char buf[];
+
 size_t writeToFileStream(void *ptr, size_t size, size_t nmemb, void *stream) {
     fwrite(ptr, size, nmemb, (FILE *) stream);
     downloaded += nmemb * size;
@@ -43,12 +49,18 @@ int downloadAndStream(char *url, void* stream, void *writeToStream) {
  //            fprintf(stderr, "download successfull, %ld\n", http_code);
              return TRUE;
          } else {
-             fprintf(stderr, "Received %ld status code for url: '%s'\n", http_code, url);
+        	 char tmp[500];
+             sprintf(tmp, "HTTP status %ld for url: '%s'\n", http_code, url);
+             fprintf(stderr, "%s\n", tmp);
+             addConsoleLine(tmp, 1, 0, 0);
              free(url);
              return FALSE;
          }
      } else {
-         fprintf(stderr, "CURL error (url = '%s'): %s\n", url, curl_easy_strerror(res));
+    	 char tmp[500];
+         sprintf(tmp, "D/l error: %s for URL: '%s'", curl_easy_strerror(res), url);
+         fprintf(stderr, "%s\n", tmp);
+         addConsoleLine(tmp, 1, 0, 0);
      }
 
      curl_easy_cleanup(curl);
@@ -64,7 +76,8 @@ int downloadAndSave(char *url, char *filename) {
     destFile = fopen(filename, "wb");
 
     if (destFile == NULL) {
-        fprintf(stderr, "unable to open file '%s' for writing\n", filename);
+        sprintf(buf, "Unable to open file '%s' for writing", filename);
+        addConsoleLine(buf, 1, 0, 0);
         return FALSE;
     }
     int result = downloadAndStream(url, destFile, &writeToFileStream);
