@@ -10,51 +10,27 @@
  *
  * Author: Damian Waradzyn
  */
-#include <sys/stat.h>
-#include <unistd.h>
-#include <libgen.h>
-#include <errno.h>
-#include <sys/time.h>
+
+#include <SDL/SDL.h>
 #include <sys/resource.h>
 
-#define ALLOC_DEBUG 0
-
-typedef enum {
-    STATE_GARBAGE = -1, STATE_EMPTY = 0, STATE_QUEUE_DELAYED = 1, STATE_QUEUED = 2, STATE_LOADING = 3, STATE_ERROR = 4,
-    // All below are guaranteed to have pixels4444 available
-    STATE_LOADED = 5,
-    STATE_SCALED = 6,
-    STATE_GL_TEXTURE_NOT_CREATED = 7,
-    STATE_LOADED_SYNC_WAIT = 8
-} TileState;
-
-typedef struct {
-    int zoom;
-    int tilex, tiley;
-    volatile GLuint texture;
-    volatile TileState state;
-    long stateChangeTime;
-    int transitionTime;
-    long textureUsedTime;
-    volatile GLubyte deleted;
-    volatile GLubyte visible;
-    GLubyte zoomHelper;
-
-    GLushort * pixels4444;
-    TileProvider* provider;
-    //char *errorMessage;
-    void * oldTiles[4];
-} t_tile;
+#include "tile.h"
+#include "main.h"
 
 t_tile* tiles[TILES_X][TILES_Y][2];
-
-#define MAX_ZOOM_HELPER_TILES 20
-
-t_tile* currentPositionZoomHelperTiles[MAX_ZOOM_HELPER_TILES];
-t_tile* destinationZoomHelperTiles[MAX_ZOOM_HELPER_TILES];
-
+t_tile* zoomTiles[MAX_ZOOM_HELPER_TILES][2];
 GQueue* allCreatedTiles = NULL;
 
+
+GQueue* getAllTilesCreated() {
+	if (allCreatedTiles == NULL) {
+		allCreatedTiles = g_queue_new();
+	}
+
+	return allCreatedTiles;
+}
+
+
 char* getTileFileName(TileProvider *provider, t_tile *tile);
 char* getTileUrl(TileProvider *provider, t_tile *tile);
 int downloadAndSave(char *url, char *filename);
@@ -66,10 +42,6 @@ void deleteElems(GQueue * queue, gpointer data);
 GQueue* garbageQueue = NULL;
 
 int isOutsideCanvas(t_tile * tile) {
-    if (tile -> zoomHelper) {
-        return FALSE;
-    }
-
     if ((tile -> tilex < canvas.tilex) || (tile -> tiley < canvas.tiley) || (tile -> tilex >= (canvas.tilex + TILES_X)) || (tile -> tiley
             >= (canvas.tiley + TILES_Y)) || (tile -> zoom != canvas.zoom) || (tile -> provider != canvas.provider)) {
         return TRUE;
@@ -77,7 +49,7 @@ int isOutsideCanvas(t_tile * tile) {
     return FALSE;
 }
 
-void deferredDeallocation(t_tile * tile) {
+void deferredDeallocation(t_tile * tile, void* unused) {
     if (tile -> state == STATE_LOADING) {
         return;
     }
@@ -85,7 +57,7 @@ void deferredDeallocation(t_tile * tile) {
     if (nowMillis - tile -> textureUsedTime < 5000) {
         return;
     }
-    if (isOutsideCanvas(tile) || tile -> deleted) {
+    if ((isOutsideCanvas(tile) && !tile -> zoomHelper) || tile -> deleted) {
         g_queue_remove_all(garbageQueue, tile);
         g_queue_push_tail(garbageQueue, tile);
     }
@@ -102,6 +74,7 @@ void processDeferredDeallocation() {
     g_queue_foreach(allCreatedTiles, (GFunc) deferredDeallocation, NULL);
 
     // Remove reffered by currently visible from garbageQueue.
+
     for (i = 0; i < TILES_X; i++) {
         for (j = 0; j < TILES_Y; j++) {
             if (tiles[i][j][currentTilesIdx] != NULL && tiles[i][j][currentTilesIdx] -> visible == TRUE) {
@@ -110,8 +83,9 @@ void processDeferredDeallocation() {
                     if ((tile -> state != STATE_LOADED) || ((tile -> state == STATE_LOADED) && nowMillis - tile->stateChangeTime
                             < tile->transitionTime)) {
                         for (k = 0; k < 4; k++) {
-                            if (tile -> oldTiles[k] != NULL) {
-                                g_queue_remove_all(garbageQueue, tile -> oldTiles[k]);
+							void* oldTile = (void*) tile -> oldTiles[k];
+                            if (oldTile != NULL) {
+                                g_queue_remove_all(garbageQueue, oldTile);
                             }
                         }
                     }
@@ -120,8 +94,9 @@ void processDeferredDeallocation() {
         }
     }
 
+
     // Deallocate all garbage.
-    //fprintf(stderr, "garbage collection (%ld ms): %d of %d tiles\n", (SDL_GetTicks() - start), g_queue_get_length(garbageQueue), g_queue_get_length(allCreatedTiles));
+//    fprintf(stderr, "garbage collection (%ld ms): %d of %d tiles\n", (SDL_GetTicks() - start), g_queue_get_length(garbageQueue), g_queue_get_length(allCreatedTiles));
     g_queue_foreach(garbageQueue, (GFunc) deallocateTile, NULL);
     g_queue_clear(garbageQueue);
     g_queue_free(garbageQueue);
@@ -166,16 +141,11 @@ void deallocateTile(t_tile *tile) {
 }
 
 // todo: move to file.c
-void ensureDirExists(char *filename) {
-    char *cpy = strdup(filename);
-    g_mkdir_with_parents(dirname(cpy), 0755);
-    free(cpy);
-}
 
-SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile);
+SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile, SDL_Surface** mask);
 
 int loadTileTexture(char* filename, t_tile* tile) {
-    SDL_Surface* texture = loadTextureFromFile(filename, tile);
+    SDL_Surface* texture = loadTextureFromFile(filename, tile, NULL);
 
     if (texture) {
         SDL_Delay(0);
@@ -194,7 +164,7 @@ int shouldAbortLoading(t_tile * tile) {
         return TRUE;
     }
 
-    if (isOutsideCanvas(tile)) {
+    if (!tile -> zoomHelper && isOutsideCanvas(tile)) {
         //        fprintf(stderr, "tile load abort, tilex = %d, tiley = %d, canvasx = %d, canvasy = %d, tile.zoom = %d, canvas.zoom = %d\n",
         //                tile -> tilex, tile -> tiley, canvas.tilex, canvas.tiley, tile -> zoom, canvas.zoom);
         tile -> state = STATE_GARBAGE;
@@ -205,19 +175,25 @@ int shouldAbortLoading(t_tile * tile) {
 
 int debugPreventLoadingTiles = 0;
 
+extern SDL_sem* sem;
+
 int downloadThread(void* queue) {
-    t_tile * tile = NULL;
-
-    setpriority(PRIO_PROCESS, 0, 19);
+    t_tile* tile = NULL;
+    int priorityStatus = 0;
+    priorityStatus = setpriority(PRIO_PROCESS, 0, 19);
 
+//    fprintf(stderr, "thread id: %p, setting priority: %d\n", (void*) SDL_ThreadID(), priorityStatus);
     while (!quit) {
         tile = (t_tile*) dequeue(queue);
         while (tile == NULL) {
             //            fprintf(stderr, "queue empty, waiting, thread id = %d\n", SDL_ThreadID());
-            SDL_Delay(0);
             if (quit) {
                 return 0;
             }
+            if (SDL_SemWait(sem) == -1) {
+                fprintf(stderr, "SemWait failed in downloadThread\n");
+                exit(1);
+            }
             tile = (t_tile*) dequeue(queue);
         }
         int mapSize = 1 << tile -> zoom;
@@ -228,6 +204,25 @@ int downloadThread(void* queue) {
             continue;
         }
 
+//        if (tile -> oldTiles[0] != NULL) {
+//        	t_tile* oldTile = (t_tile*) tile -> oldTiles[0];
+//        	if (!isOutsideCanvas(oldTile)) {
+//        		tile -> texture = oldTile -> texture;
+//        		tile -> pixels4444 = oldTile -> pixels4444;
+//
+//        		oldTile -> texture = 0;
+//        		oldTile -> pixels4444 = NULL;
+//        		oldTile -> deleted = TRUE;
+//        		oldTile -> state = STATE_GARBAGE;
+//
+//        		tile -> transitionTime = 0;
+//        		tile -> state = STATE_LOADED;
+//                        fprintf(stderr, "tile not necessary, tilex = %d, tiley = %d, canvasx = %d, canvasy = %d, tile.zoom = %d, canvas.zoom = %d\n",
+//                                tile -> tilex, tile -> tiley, canvas.tilex, canvas.tiley, tile -> zoom, canvas.zoom);
+//        		continue;
+//        	}
+//        }
+
         if (shouldAbortLoading(tile)) {
             continue;
         }
@@ -241,7 +236,7 @@ int downloadThread(void* queue) {
 
             if (shouldAbortLoading(tile)) {
                 free(filename);
-                continue;
+                continue; 
             }
 
             //            fprintf(stderr, "load tile from local file failed, trying to download tile\n");
